﻿using System;

/* Raima.Rdm is the system module that contains the ADO.NET
 * data provider for RDM */
using Raima.Rdm;

/* EXAMPLE - SQL07 for C# using ADO.NET */
/* This example shows how to use parameters with SQL commands.  A database
 * is created and then data is inserted with positioned parameters and then
 * with named parameters.  Finally the data is read out and displayed.
 * 
 * The following methods are illustrated:
 *     Raima.Rdm.RdmConnection:
 *         ChangeDatabase(string) Method
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection() Constructor
 *     Raima.Rdm.RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteReader() Method
 *         Parameters Property
 *     Raima.Rdm.RdmDataReader:
 *         Item[int] Property
 *         Read() Method
 *     Raima.Rdm.RdmParameterCollection
 *         Add(RdmParameter) Method
 *         Add(String, RdmType) Method
 *         Clear() Method
 *         Item[int] Property
 *         Item[string] Property
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown execptions
 * but good programming practices would dictate that this occur in
 * a full application.
 */

namespace SQL07
{
    class Program
    {
        static void DropDatabase(
            RdmConnection conn,
            string name)
        {
            RdmCommand cmd = conn.CreateCommand();

            /* Attempt to drop this database.  If the database does not
             * already exist this will fail and we simply ignore the
             * failure for this sample. */
            try
            {
                cmd.CommandText = "DROP DATABASE " + name;
                cmd.ExecuteNonQuery();

                cmd.CommandText = "COMMIT";
                cmd.ExecuteNonQuery();
            }
            catch (Exception)
            {
            }
        }

        static void CreateThisDatabase(
            RdmConnection conn,
            string        name,
            string[]      cmds)
        {
            RdmCommand cmd = conn.CreateCommand();

            /* Create the specified database.  Make sure the database is
             * closed when we are done so other databases can be created
             * after this if necessary. */
            cmd.CommandText = "CREATE DATABASE " + name;
            cmd.ExecuteNonQuery();

            foreach (string c in cmds)
            {
                cmd.CommandText = c;
                cmd.ExecuteNonQuery();
            }

            cmd.CommandText = "COMMIT";
            cmd.ExecuteNonQuery();

            cmd.CommandText = "CLOSE " + name;
            cmd.ExecuteNonQuery();
        }

        static void CreateDatabases(
            RdmConnection conn)
        {
            string[] cmds =
            {
                "CREATE TABLE Table1 (id INTEGER PRIMARY KEY, str CHAR(30))"
            };

            /* Drop any existing copy of the database */
            DropDatabase(conn, "SQL07");

            /* Attempt to create the database */
            CreateThisDatabase(conn, "SQL07", cmds);
        }

        static void Main(string[] args)
        {
            /* Create the connection with a simple connection string */
            RdmConnection conn = new RdmConnection("host=local;openmode=x");
            conn.Open();

            CreateDatabases(conn);

            /* Open the database */
            conn.ChangeDatabase("SQL07");

            /* Create a new command */
            RdmCommand cmd = conn.CreateCommand();

            /* Insert three rows into the table using positioned parameters */
            cmd.CommandText = "INSERT INTO Table1 VALUES ?, ?";
            cmd.Parameters.Add(new RdmParameter());
            cmd.Parameters.Add(new RdmParameter());

            for (int ii = 1; ii < 4; ii++)
            {
                cmd.Parameters[0].Value = ii;
                cmd.Parameters[1].Value = "positioned " + ii;
                cmd.ExecuteNonQuery();
            }

            /* Insert three rows into the table using named parameters */
            cmd.Parameters.Clear();
            cmd.CommandText = "INSERT INTO Table1 VALUES :idparm, :strparm";
            cmd.Parameters.Add("idparm", RdmType.Int32);
            cmd.Parameters.Add("strparm", RdmType.AnsiString);

            for (int ii = 4; ii < 7; ii++)
            {
                cmd.Parameters["idparm"].Value = ii;
                cmd.Parameters["strparm"].Value = "named " + ii;
                cmd.ExecuteNonQuery();
            }

            /* Scan through the data, displaying the rows */
            cmd.Parameters.Clear();
            cmd.CommandText = "SELECT id, str FROM Table1";
            RdmDataReader reader = cmd.ExecuteReader();

            while (reader.Read())
                Console.WriteLine(reader[0] + " -- " + reader[1]);

            conn.Close();

            Console.WriteLine("Hit <Enter> to continue");
            Console.ReadLine();
        }
    }
}
